@@ -7,7 +7,7 @@
 *
 * FOCAL - A FOCAL language interpreter
 *
-* Copyright (c) 1978-2009 Dave Pitts <dpitts@cozx.com>
+* Copyright (c) 1978-2011 Dave Pitts <dpitts@cozx.com>
 *
 * FOCAL is language developed by Digital Equipment Corporation (DEC).
 * To learn FOCAL syntax refer to DEC Programming Languages Handbook:
@@ -32,7 +32,7 @@
 *
 ***********************************************************************/
 
-#define VERSION "1.0.1"
+#define VERSION "1.0.6"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -69,14 +69,25 @@
 #define STDERROUT	"\n"
 #endif /* VAXVMS */
 
+#if defined(DX10)
+#include <sys/svc.h>
+#endif
+
 #include "parser.h"
 #include "errors.h"
  
 #define EOL       '\0'                  /* end of line */
-#define LINE_LEN  132                   /* source line length */
 #define TWOCHAR   2                     /* group/step sizes */
 #define PLEN      132                   /* print buffer length */
 #define PBEG      2                     /* print buffer start */
+
+#ifdef DX10
+#define LINE_LEN  80                    /* source line length */
+#define PATH_LEN  255                   /* path name length */
+#else
+#define LINE_LEN  132                   /* source line length */
+#define PATH_LEN  1024                  /* path name length */
+#endif
  
 #define DO_FLG    1
 #define FOR_FLG   2
@@ -133,6 +144,7 @@ typedef struct for_stk
    struct for_stk *for_ptr;
    tokval          for_inc;
    tokval          for_limit;
+   tokval          for_subscript;
    char            for_index[TWOCHAR+1];
 } for_stk_t;
  
@@ -153,11 +165,13 @@ static sym_node_t  *sym_anchor;         /* symbol table anchor */
 static pc_stk_t    *pc_top;             /* top of the PC stack */
 static for_stk_t   *for_top;            /* top of the FOR loop stack */
 static do_stk_t    *do_top;             /* top of the DO routine stack */
+#ifndef SMALL_MEMORY
 static line_node_t *free_line;          /* free lines */
 static sym_node_t  *free_sym;           /* free symbols */
 static for_stk_t   *free_for;           /* free for */
 static pc_stk_t    *free_pc;            /* free pcs */
 static do_stk_t    *free_do;            /* free dos */
+#endif
  
 static int run_mode;                    /* program running mode 'GO' */
 static int do_mode;                     /* program in subroutine 'DO' */
@@ -171,8 +185,8 @@ static int digits;                      /* print field significance */
 static long seed;                       /* random number seed */
  
 static char *pcp;                       /* pointer into current PC line */
-static char tbuf[LINE_LEN+1];
-static char wd[LINE_LEN+1];
+static char tbuf[LINE_LEN+2];
+static char wd[PATH_LEN+1];
  
 static FILE *input;
 static FILE *output;
@@ -226,6 +240,11 @@ extern void screenposition (char *, char*);
 int err_flag;
 int fpe_stop;                           /* floating point exception */
 
+/* some toupper functions don't check case first */
+#ifndef NOFOLD
+#define upcase toupper
+#endif
+
 #ifdef DEBUG_FILE
 /***********************************************************************
 * HEXDUMP
@@ -294,6 +313,28 @@ HEXDUMP (FILE *file, void *ptr, int size)
 }
 #endif
 
+#if defined(DX10)
+/***********************************************************************
+* checkterm = check for terminal event.
+***********************************************************************/
+
+void
+checkterm (void)
+{
+   SVCEvtBlk svc;
+
+   svc.opcode = 0x39;
+   svc.status = 0;
+   svc.event = 0;
+   svc.luno = 0;
+
+   _issue_svc ((SVCBlk *)&svc);
+   if (svc.status == 0 && svc.event == 0x98)
+      user_stop = TRUE;
+}
+#endif
+
+#ifdef NOFOLD
 /***********************************************************************
 * upcase
 ***********************************************************************/
@@ -305,6 +346,7 @@ upcase (char ch)
       ch = toupper (ch);
    return (ch);
 }
+#endif
  
 /***********************************************************************
 * freedo - free a DO element
@@ -313,8 +355,12 @@ upcase (char ch)
 static void
 freedo (do_stk_t *p)
 {
+#ifdef SMALL_MEMORY
+   free (p);
+#else
    p->do_ptr = free_do;
    free_do = p;
+#endif
 } /* freedo */
  
 /***********************************************************************
@@ -324,8 +370,12 @@ freedo (do_stk_t *p)
 static void
 freefor (for_stk_t *p)
 {
+#ifdef SMALL_MEMORY
+   free (p);
+#else
    p->for_ptr = free_for;
    free_for = p;
+#endif
 } /* freefor */
  
 /***********************************************************************
@@ -335,8 +385,12 @@ freefor (for_stk_t *p)
 static void
 freeline (line_node_t *p)
 {
+#ifdef SMALL_MEMORY
+   free (p);
+#else
    p->line_ptr = free_line;
    free_line = p;
+#endif
 } /* freeline */
  
 /***********************************************************************
@@ -346,8 +400,12 @@ freeline (line_node_t *p)
 static void
 freesym (sym_node_t *p)
 {
+#ifdef SMALL_MEMORY
+   free (p);
+#else
    p->sym_ptr = free_sym;
    free_sym = p;
+#endif
 } /* freesym */
  
 /************************************************************************
@@ -374,8 +432,7 @@ askcmd (void)
  
    /* Position to the next field */
    nextfield();
-   if (output == stdout)
-      fflush (output);
+   fflush (output);
  
    do
    {
@@ -400,15 +457,22 @@ askcmd (void)
          fputs (":", stdout);
 #if defined(OS390) || defined(OS2)
 	 fputc ('\n', stdout);
+#endif
+#if defined(OS390) || defined(OS2) || defined(DX10)
          fflush (stdout);
 #endif
-	 if (output == stdout)
-	    pndx = PBEG;
+         pndx = PBEG;
  
          /* Read input */
-         fgets (tbuf, sizeof (tbuf), stdin);
-         if (tbuf[0] != '\n')
-            val = Parser (tbuf);
+	 tbuf[0] = '0';
+         if ((fgets (&tbuf[1], sizeof (tbuf) - 1, stdin) != NULL) &&
+	       (tbuf[1] != '\n'))
+	 {
+	    if (isalpha (tbuf[1]))
+	       val = Parser (tbuf);
+	    else
+	       val = Parser (&tbuf[1]);
+	 }
          else
             val = 0.0;
  
@@ -421,8 +485,7 @@ askcmd (void)
       {
          pcp++;
          fputc ('\n', stdout);
-	 if (output == stdout)
-	    pndx = PBEG;
+         pndx = PBEG;
       }
  
       /* check for prompt text */
@@ -445,6 +508,7 @@ askcmd (void)
 	 }
       }
  
+#if defined(ANSICRT)
       /* Position on CRT */
       else if (*pcp == '@')
       {
@@ -455,9 +519,7 @@ askcmd (void)
          if (upcase (*pcp) == 'E')
          {
             pcp++;
-#if defined(ANSICRT)
             clearscreen();
-#endif
          }
          if (isdigit (*pcp))
          {
@@ -467,18 +529,15 @@ askcmd (void)
                pcp++;
                getgrp (col);
             }
-#if defined(ANSICRT)
             screenposition (row, col);
-#endif
          }
          if (upcase (*pcp) == 'C')
          {
             pcp++;
-#if defined(ANSICRT)
             clearline();
-#endif
          }
       }
+#endif
  
       else if (*pcp == EOL || *pcp == ';') ;
  
@@ -577,6 +636,12 @@ docmd (void)
       nextfield();
    }
  
+#ifdef SMALL_MEMORY
+   if ((p = (do_stk_t *)malloc (sizeof (do_stk_t))) == NULL)
+   {
+      error (MEM_OVERFLOW, STACK_OVERFLOW);
+   }
+#else
    if (free_do == NULL)
    {
       if ((p = (do_stk_t *)malloc (sizeof (do_stk_t))) == NULL)
@@ -589,11 +654,12 @@ docmd (void)
       p = free_do;
       free_do = p->do_ptr;
    }
+#endif
  
    if (p != NULL)
    {
  
-      memset ((void *)p, '\0', sizeof (do_stk_t));
+      memset ((void *)p, 0, sizeof (do_stk_t));
 
       /* Get group from buffer */
       getgrp (p->do_grp);
@@ -834,20 +900,17 @@ error (int err_code, int err_stat)
          strcpy (errorstring, "Stop");
          break;
       
+#if !defined(DX10)
       case FPE_STOP:
          fpe_stop = FALSE;
          strcpy (errorstring, "Floating point exception");
          break;
+#endif
  
       case BAD_CMD:
          strcpy (errorstring, "Bad command");
-         switch (err_stat)
-         {
-            case LIBRARY_CMD :
-               strcat (errorstring, " in Library");
-	       break;
-            default: ;
-         }
+         if (err_stat == LIBRARY_CMD)
+	    strcat (errorstring, " in Library");
 	 strcat (errorstring, "\nType HELP for a list of commands");
          break;
  
@@ -975,7 +1038,7 @@ error (int err_code, int err_stat)
          strcpy (errorstring, "Undefined error");
    }
 
-   fprintf (stderr, errorstring);
+   fputs (errorstring, stderr);
  
    /* clean up */
    cleaner();
@@ -1062,11 +1125,16 @@ execline (void)
                error (BAD_CMD, COMMAND);
          }
  
+#if !defined(DX10)
          /* Check for floating point exception */
          if (fpe_stop)
             error (FPE_STOP, 0);
+#endif
  
          /* Check for user abort */
+#if defined(DX10)
+         checkterm();
+#endif
          if (user_stop)
             error (USER_STOP, 0);
  
@@ -1079,9 +1147,11 @@ execline (void)
       {
          if (pc_top->pc_flags == FOR_FLG)
          {
-            symboltable (for_top->for_index, &val, 0, TRUE);
+            symboltable (for_top->for_index, &val,
+			 for_top->for_subscript, TRUE);
             val = val + for_top->for_inc;
-            symboltable (for_top->for_index, &val, 0, FALSE);
+            symboltable (for_top->for_index, &val,
+			 for_top->for_subscript, FALSE);
             if (val <= for_top->for_limit)
             {
                next = FALSE;
@@ -1251,23 +1321,13 @@ findline (void)
 static void
 fmtnum (tokval val)
 {
-   int ival;
    char format[30];
  
    if (width > 0)
    {
       pndx += width;
-      if (digits > 0)
-      {
-         sprintf (format, "= %%%d.%df", width, digits);
-         fprintf (output, format, val);
-      }
-      else
-      {
-         sprintf (format, "= %%%dd", width);
-         ival = val;
-         fprintf (output, format, ival);
-      }
+      sprintf (format, "= %%%d.%df", width, digits);
+      fprintf (output, format, val);
    }
  
    else
@@ -1292,6 +1352,7 @@ forcmd (void)
 {
    for_stk_t *p;
    tokval v1, val;
+   int ndx;
  
    /* Skip to index variable */
    nextfield();
@@ -1303,6 +1364,12 @@ forcmd (void)
       return;
    }
  
+#ifdef SMALL_MEMORY
+   if ((p = (for_stk_t *)malloc (sizeof (for_stk_t))) == NULL)
+   {
+      error (MEM_OVERFLOW, STACK_OVERFLOW);
+   }
+#else
    if (free_for == NULL)
    {
       if ((p = (for_stk_t *)malloc (sizeof (for_stk_t))) == NULL)
@@ -1315,17 +1382,27 @@ forcmd (void)
       p = free_for;
       free_for = p->for_ptr;
    }
+#endif
  
    if (p != NULL)
    {
  
-      memset ((void *)p, '\0', sizeof (for_stk_t));
+      memset ((void *)p, 0, sizeof (for_stk_t));
 
       /* Get index symbol */
       getsym (p->for_index);
  
       if (*pcp == ' ')
          nextfield();
+
+      ndx = 0;
+      if (*pcp == '(' || *pcp == '<' || *pcp == '[' || *pcp == '{')
+      {
+	 ndx = expression();
+	 if (*pcp == ' ')
+	    nextfield();
+      }
+      p->for_subscript = ndx;
  
       if (*pcp == '=')
       {
@@ -1333,7 +1410,7 @@ forcmd (void)
          /* get start expression */
          pcp++;
          v1 = val = expression();
-         symboltable (p->for_index, &val, 0, FALSE);
+         symboltable (p->for_index, &val, p->for_subscript, FALSE);
  
          /* Get increment */
          pcp++;
@@ -1355,7 +1432,7 @@ forcmd (void)
             p->for_limit = expression();
          }
  
-         if (!err_flag && (p->for_limit >= v1))
+         if (!err_flag)
          {
             /* Set up FOR stack */
             p->for_ptr = for_top;
@@ -1556,6 +1633,7 @@ gotoit (void)
 static void
 helpcmd (void)
 {
+#if !defined(MINFOCAL)
    int all = FALSE;
 
    /* Position to command, if any */
@@ -1578,7 +1656,7 @@ helpcmd (void)
 	    printf ("   D[O] ['?'] <GROUP>[.<STEP>]\n");
 	    if (!all) break;
 	 case 'E' :
-	    printf ("   E[RASE] [A[LL] | <GROUP>[.<STEP]]\n"); 
+	    printf ("   E[RASE] [A[LL] | <GROUP>[.<STEP>]]\n"); 
 	    if (!all) break;
 	 case 'F' :
 	    printf ("   F[OR] <VAR>=<START>[,<INC>],<END>; <COMMAND>\n");
@@ -1594,8 +1672,8 @@ helpcmd (void)
 	    if (!all) break;
 	 case 'L' :
 	    printf ("   L[IBRARY] C[ALL] <FILENAME>\n");
-#if defined(BSDDIR) || defined(SYSVDIR)
-	    printf ("   L[IBRARY] L[IST] [<PATHNAME>]\n");
+#if defined(BSDDIR) || defined(SYSVDIR) || defined(DOS)
+	    printf ("   L[IBRARY] L[IST] [<DIRNAME>]\n");
 #endif
 	    printf ("   L[IBRARY] P[RINT] <FILENAME>\n");
 	    printf ("   L[IBRARY] S[AVE] <FILENAME>\n");
@@ -1619,7 +1697,7 @@ helpcmd (void)
 	    printf ("   T[YPE] [\"TEXT\",] <VAR> | <EXPRESSION>\n");
 	    if (!all) break;
 	 case 'W' :
-	    printf ("   W[RITE] [A[LL] | <GROUP>[.<STEP]]\n"); 
+	    printf ("   W[RITE] [A[LL] | <GROUP>[.<STEP>]]\n"); 
 	    if (!all) break;
 	 case ' ' :
 	    break;
@@ -1629,6 +1707,9 @@ helpcmd (void)
       nextfield();
    }
    else
+#else
+   while (*pcp != EOL && *pcp != ';') pcp++;
+#endif
    {
       printf ("Commands:\n");
       printf ("   ASK      CONTINUE DO       ERASE    FOR      GOTO\n");
@@ -1716,6 +1797,12 @@ insertline (void)
    int found;
    char olinnum[6], nlinnum[6];
  
+#ifdef SMALL_MEMORY
+   if ((p = (line_node_t *)malloc (sizeof (line_node_t))) == NULL)
+   {
+      error (MEM_OVERFLOW, LINE_OVERFLOW);
+   }
+#else
    if (free_line == NULL)
    {
       if ((p = (line_node_t *)malloc (sizeof (line_node_t))) == NULL)
@@ -1728,10 +1815,11 @@ insertline (void)
       p = free_line;
       free_line = p->line_ptr;
    }
+#endif
  
    if (p != NULL)
    {
-      memset ((void *)p, '\0', sizeof (line_node_t));
+      memset ((void *)p, 0, sizeof (line_node_t));
 
       p->line_ptr = NULL;
       /* Get line group number */
@@ -1882,7 +1970,7 @@ librarycmd (void)
    line_node_t *spc, *p, tp;
    char ch, *bp, *mode;
    int done, k;
-   char fname[LINE_LEN+1];
+   char fname[PATH_LEN+1];
  
    /* Position to function */
    nextfield();
@@ -1907,11 +1995,12 @@ librarycmd (void)
       case 'C' : /* Call (load) program */
          if (tbuf[0] != EOL)
          {
-#if defined(BSDDIR) || defined(SYSVDIR)
+#if defined(DX10)
+	    if (wd[0])
+	       sprintf (fname, "%s.", wd);
+#elif defined(BSDDIR) || defined(SYSVDIR)
 	    if (tbuf[0] != '/')
-	    {
 	       sprintf (fname, "%s/", wd);
-	    }
 #endif
 	    strcat (fname, tbuf);
          }
@@ -1924,7 +2013,7 @@ librarycmd (void)
 #if defined(OS390)
          for (bp = fname; *bp; bp++)
 	    *bp = upcase (*bp);
-#else
+#elif !defined(DX10)
          /* Append extension */
  
          if ((bp = (char *)strrchr (tbuf, '.')) == NULL)
@@ -1943,12 +2032,13 @@ librarycmd (void)
 
                if (fgets (tbuf, sizeof (tbuf), progfile) != NULL)
                {
-		  *((char *)(strchr (tbuf, '\n'))) = EOL;
+		  if ((bp = strchr (tbuf, '\n')) != NULL)
+		     *bp = EOL;
                   k = strlen (tbuf);
 #ifdef DEBUG_FILE
                   printf ("tbuf(%d) = '%s'\n", k, tbuf);
 #endif
-		  memset ((void *)&tp, '\0', sizeof(line_node_t));
+		  memset ((void *)&tp, 0, sizeof(line_node_t));
                   pc = &tp;
                   pcp = pc->line_txt;
                   strcpy (pcp, tbuf);
@@ -1975,14 +2065,11 @@ librarycmd (void)
          break;
  
       case 'L' : /* L(ist) files */
-#if defined(BSDDIR) || defined(SYSVDIR)
+#if (defined(BSDDIR) || defined(SYSVDIR)) && !defined(MINFOCAL)
          if (tbuf[0] == EOL)
-         {
             strcpy (tbuf, wd);
-         }
          if ((dir = opendir (tbuf)) != NULL)
          {
-            struct stat stbuf;
 #if defined(BSDDIR)
             struct direct *myent;
 #endif
@@ -1991,12 +2078,17 @@ librarycmd (void)
 #endif
             while ((myent = readdir (dir)) != NULL)
             {
+#ifdef DX10
+	       if ((myent->d_flags & 0xC000) == 0)
+		  printf (" %s\n", myent->d_name);
+#else
                char *bp;
 
                if ((bp = strrchr (myent->d_name, '.')) != NULL)
                {
                   if (!strcmp (bp, FOCAL_EXT))
 		  {
+		     struct stat stbuf;
 
 		     sprintf (fname, "%s/%s", tbuf, myent->d_name);
 		     if (stat (fname, &stbuf) == 0)
@@ -2015,6 +2107,7 @@ librarycmd (void)
 		     }
 		  }
                }
+#endif
             }
             closedir (dir);
          }
@@ -2028,6 +2121,8 @@ librarycmd (void)
 #endif
             error (FILE_ERROR, errno);
          }
+#elif defined(DOS)
+         system ("dir /O/4 *.foc");
 #else
          fprintf (stderr, "List not supported for this OS");
 	 error (FILE_ERROR, 0);
@@ -2054,11 +2149,12 @@ librarycmd (void)
          {
 	    if (tbuf[0] != EOL)
 	    {
-#if defined(BSDDIR) || defined(SYSVDIR)
+#if defined(DX10)
+	       if (wd[0])
+		  sprintf (fname, "%s.", wd);
+#elif defined(BSDDIR) || defined(SYSVDIR)
 	       if (tbuf[0] != '/')
-	       {
 		  sprintf (fname, "%s/", wd);
-	       }
 #endif
 	       strcat (fname, tbuf);
 	    }
@@ -2071,7 +2167,7 @@ librarycmd (void)
 #if defined(OS390)
 	    for (bp = fname; *bp; bp++)
 	       *bp = upcase(*bp);
-#else
+#elif !defined(DX10)
 	    /* Append extension */
     
 	    if ((bp = (char *)strrchr (tbuf, '.')) == NULL)
@@ -2097,11 +2193,12 @@ librarycmd (void)
       case 'S' : /* Save program */
          if (tbuf[0] != EOL)
          {
-#if defined(BSDDIR) || defined(SYSVDIR)
+#if defined(DX10)
+	    if (wd[0])
+	       sprintf (fname, "%s.", wd);
+#elif defined(BSDDIR) || defined(SYSVDIR)
 	    if (tbuf[0] != '/')
-	    {
 	       sprintf (fname, "%s/", wd);
-	    }
 #endif
 	    strcat (fname, tbuf);
          }
@@ -2114,7 +2211,7 @@ librarycmd (void)
 #if defined(OS390)
          for (bp = fname; *bp; bp++)
 	    *bp = upcase(*bp);
-#else
+#elif !defined(DX10)
          /* Append extension */
  
          if ((bp = (char *)strrchr (tbuf, '.')) == NULL)
@@ -2147,9 +2244,12 @@ librarycmd (void)
          break;
  
       case 'W': /* Change Working directory */
-#if defined(BSDDIR) || defined(SYSVDIR)
+#if (defined(BSDDIR) || defined(SYSVDIR)) && !defined(MINFOCAL)
 	 if (tbuf[0] != EOL)
 	 {
+#if defined(DX10)
+	    strcpy (wd, tbuf);
+#else
 	    if ((dir = opendir (tbuf)) != NULL)
 	    {
 	       closedir (dir);
@@ -2165,10 +2265,11 @@ librarycmd (void)
 #endif
 	       error (FILE_ERROR, errno);
 	    }
+#endif
 	 }
 	 else
 	 {
-	    fprintf (stderr, "Pathname is required");
+	    fprintf (stderr, "Directory is required");
 	    error (FILE_ERROR, 0);
 	 }
 #else
@@ -2318,8 +2419,12 @@ pcpop (void)
  
    p = pc_top;
    pc_top = p->pc_ptr;
+#ifdef SMALL_MEMORY
+   free (p);
+#else
    p->pc_ptr = free_pc;
    free_pc = p;
+#endif
  
 } /* pcpop */
  
@@ -2336,6 +2441,12 @@ pcpush (void)
 {
    pc_stk_t *p;
  
+#ifdef SMALL_MEMORY
+   if ((p = (pc_stk_t *)malloc (sizeof (pc_stk_t))) == NULL)
+   {
+      error (MEM_OVERFLOW, STACK_OVERFLOW);
+   }
+#else
    if (free_pc == NULL)
    {
       if ((p = (pc_stk_t *)malloc (sizeof (pc_stk_t))) == NULL)
@@ -2348,10 +2459,11 @@ pcpush (void)
       p = free_pc;
       free_pc = p->pc_ptr;
    }
+#endif
  
    if (p != NULL)
    {
-      memset ((void *)p, '\0', sizeof (pc_stk_t));
+      memset ((void *)p, 0, sizeof (pc_stk_t));
 
       p->pc_index = pcp;
       p->old_pc = pc;
@@ -2518,6 +2630,12 @@ symboltable (char *sym, tokval *val, int ndx, int flg)
       if (j < TWOCHAR)
          lsym[j] = sym[j];
  
+#ifdef SMALL_MEMORY
+   if ((p = (sym_node_t *)malloc (sizeof (sym_node_t))) == NULL)
+   {
+      error (MEM_OVERFLOW, SYMBOL_OVERFLOW);
+   }
+#else
    if (free_sym == NULL)
    {
       if ((p = (sym_node_t *)malloc (sizeof (sym_node_t))) == NULL)
@@ -2530,10 +2648,11 @@ symboltable (char *sym, tokval *val, int ndx, int flg)
       p = free_sym;
       free_sym = p->sym_ptr;
    }
+#endif
  
    if (p != NULL)
    {
-      memset ((void *)p, '\0', sizeof (sym_node_t));
+      memset ((void *)p, 0, sizeof (sym_node_t));
 
       /* Initialize new node */
       p->sym_ptr = NULL;
@@ -2672,6 +2791,9 @@ typecmd (void)
             p = sym_anchor;
             while (p != NULL)
             {
+#if defined(DX10)
+               checkterm();
+#endif
                if (user_stop)
                {
                   user_stop = FALSE;
@@ -2710,6 +2832,7 @@ typecmd (void)
             pndx = k;
             break;
  
+#if defined(ANSICRT)
          case '@' : /* Position on the screen */
             pcp++;
             strcpy (row, "01");
@@ -2718,9 +2841,7 @@ typecmd (void)
             if (upcase (*pcp) == 'E')
             {
                pcp++;
-#if defined(ANSICRT)
                clearscreen();
-#endif
             }
  
             if (isdigit (*pcp))
@@ -2731,19 +2852,16 @@ typecmd (void)
                   pcp++;
                   getgrp (col);
                }
-#if defined(ANSICRT)
                screenposition (row, col);
-#endif
             }
  
             if (upcase (*pcp) == 'C')
             {
                pcp++;
-#if defined(ANSICRT)
                clearline();
-#endif
             }
             break;
+#endif
  
          case ' ' :
          case ',' :
@@ -2800,6 +2918,9 @@ writecmd (void)
  
       while (p != NULL)
       {
+#if defined(DX10)
+	 checkterm();
+#endif
          if (user_stop)
          {
             user_stop = FALSE;
@@ -2863,6 +2984,9 @@ writecmd (void)
                {
                   fprintf (output, "%s.%s %s\n",
 			   p->grp_num, p->stp_num, p->line_txt);
+#if defined(DX10)
+		  checkterm();
+#endif
                   if (user_stop)
                   {
                      user_stop = FALSE;
@@ -2880,6 +3004,7 @@ writecmd (void)
  
 } /* writecmd */
  
+#if !defined(DX10)
 /***********************************************************************
 * fpeint - floating point exception
 ***********************************************************************/
@@ -2901,6 +3026,7 @@ userint (int sig)
    user_stop = TRUE;
    signal (SIGINT, userint);
 }
+#endif
  
 /**********************************************************************
 *
@@ -2912,7 +3038,6 @@ int
 main (int argc, char *argv[])
 {
  
- 
    /* Index into print buffer */
    output = stdout;
    input = stdin;
@@ -2934,11 +3059,13 @@ main (int argc, char *argv[])
    for_top = NULL;
    pc_top = NULL;
    do_top = NULL;
+#ifndef SMALL_MEMORY
    free_line = NULL;
    free_sym = NULL;
    free_for = NULL;
    free_pc = NULL;
    free_do = NULL;
+#endif
  
    /* Set initial flags */
    quit_flag = FALSE;
@@ -2949,16 +3076,22 @@ main (int argc, char *argv[])
  
    /* Allocate keyboard buffer */
    buffer = (line_node_t *)malloc (sizeof (line_node_t));
-   memset ((void *)buffer, '\0', sizeof (line_node_t));
+   memset ((void *)buffer, 0, sizeof (line_node_t));
  
+#if !defined(DX10)
    /* Initialize keyboard event */
    signal (SIGINT, userint);
 
    /* Initialize Floating point exception */
    signal (SIGFPE, fpeint);
+#endif
 
    /* Set the working directory */
+#if defined(DX10)
+   wd[0] = '\0';
+#else
    strcpy (wd, ".");
+#endif
  
    /* If an arg, then execute it */
    if (argc == 2)
@@ -2989,6 +3122,8 @@ main (int argc, char *argv[])
       fputc ('*', stdout);
 #if defined(OS390) || defined(OS2)
       fputc ('\n', stdout);
+#endif
+#if defined(OS390) || defined(OS2) || defined(DX10)
       fflush (stdout);
 #endif
  
